unit CommonMemory;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
   Dialogs, ExtCtrls;

type
  PByte = ^Byte; // 8 bit
  PWord = ^Word; // 16 bit
  PInteger = ^Integer; // 32 bit
  PDouble = ^Double; // 64 bit

  { TCommonMemory NX }
  TCommonMemory = class(TObject)
  private
    { Private 錾 }
    FHandle: Integer; // L̃nh
    FMemory: Pointer; // L
    FSize: Integer; // L̃TCY
    FName: String; // L̖O
    FStr: String; // L̃f[^̕\
    FStrList: TStringList; // ̓͂̕T|[g
    FProcessCount: Integer; // ݂̋Lgp̃vZX
    function GetStr: String;
    function GetVByte(Index: Integer): Byte;
    function GetVWord(Index: Integer): Word;
    function GetVInteger(Index: Integer): Integer;
    function GetVDouble(Index: Integer): Double;
    procedure SetStr(const Value: String);
    procedure SetVByte(Index: Integer; const Value: Byte);
    procedure SetVWord(Index: Integer; const Value: Word);
    procedure SetVInteger(Index: Integer; const Value: Integer);
    procedure SetVDouble(Index: Integer; const Value: Double);
  protected
    { Protected 錾 }
  public
    { Public 錾 }
    constructor Create(const Name: String; Size: Longint);
    destructor Destroy; override;
    procedure Clear; virtual; // f[^[NA[B
    { vpeB錾 }
    property Handle: Integer read FHandle;
    property Memory: Pointer read FMemory;
    property Size: Integer read FSize;
    property Name: String read FName;
    property Str: String read GetStr write SetStr;
    property StrList: TStringList read FStrList;
    property ProcessCount: Integer read FProcessCount;
    property VByte[Index: Integer]: Byte read GetVByte write SetVByte;
    property VWord[Index: Integer]: Word read GetVWord write SetVWord;
    property VInteger[Index: Integer]: Integer read GetVInteger write SetVInteger; default;
    property VDouble[Index: Integer]: Double read GetVDouble write SetVDouble;
end;

implementation

{ TPreviewBox }

constructor TCommonMemory.Create(const Name: String; Size: Longint);
var
  P: PInteger;
begin
  inherited Create;

  FHandle := 0;
  FMemory := Nil;
  FSize := 0;
  FName := Name;
  FStr := '';
  FStrList := TStringList.Create;
  FStrList.Duplicates := dupAccept;
  FProcessCount := 0;

  FHandle := OpenFileMapping(File_MAP_WRITE,
                             False,
                             PChar(FName));

  if FHandle = 0 then
    FHandle := CreateFileMapping($FFFFFFFF,
                                 Nil,
                                 PAGE_READWRITE,
                                 0,
                                 Size + 4, // FProcessCount ̕TCY𑝂₷B
                                 PChar(FName));

  if FHandle <> 0 then
    FMemory := MapViewOfFile(FHandle,
                             FILE_MAP_WRITE,
                             0,
                             0,
                             0);

  if FMemory <> Nil then
  begin
    FSize := Size;
    Longint(P) := Longint(FMemory) + FSize;
    FProcessCount := P^;
    Inc(FProcessCount); // gp̃vZX1₷B
    P^ := FProcessCount;
  end;
end;

destructor TCommonMemory.Destroy;
var
  P: PInteger;
begin
  Longint(P) := Longint(FMemory) + FSize;
  FProcessCount := P^;
  Dec(FProcessCount); // gp̃vZX1炷B
  P^ := FProcessCount;
  if FProcessCount < 1 then
  begin
    UnMapViewOfFile(FMemory); // }bsOB
    CloseHandle(FHandle); // L̃nhB
  end;
  FStrList.Free;
  inherited Destroy;
end;

procedure TCommonMemory.Clear;
begin
  FStr := '';
  FStrList.Clear;
  ZeroMemory(FMemory, FSize);
end;  

function TCommonMemory.GetStr: String;
begin
  FStr := PChar(FMemory); // f[^̕\
  Result := FStr;
end;

function TCommonMemory.GetVByte(Index: Integer): Byte;
var
  P: PByte;
begin
  if (Index < 0) or (Index >= FSize) then
    raise ERangeError.CreateFmt('Index = %d is not within the valid range of %d..%d',
     [Index, 0, FSize - 1]);

  Longint(P) := Longint(FMemory) + Index;
  Result := P^;
end;

function TCommonMemory.GetVDouble(Index: Integer): Double;
var
  i: Integer;
  P: PDouble;
begin
  i := FSize div SizeOf(Double);
  if (Index < 0) or (Index > i) then
    raise ERangeError.CreateFmt('Index = %d is not within the valid range of %d..%d',
     [Index, 0, i]);

  Longint(P) := Longint(FMemory) + SizeOf(Double) * Index;
  Result := P^;
end;

function TCommonMemory.GetVInteger(Index: Integer): Integer;
var
  i: Integer;
  P: PInteger;
begin
  i := FSize div SizeOf(Integer);
  if (Index < 0) or (Index > i) then
    raise ERangeError.CreateFmt('Index = %d is not within the valid range of %d..%d',
     [Index, 0, i]);

  Longint(P) := Longint(FMemory) + SizeOf(Integer) * Index;
  Result := P^;
end;

function TCommonMemory.GetVWord(Index: Integer): Word;
var
  i: Integer;
  P: PWord;
begin
  i := FSize div SizeOf(Word);
  if (Index < 0) or (Index > i) then
    raise ERangeError.CreateFmt('Index = %d is not within the valid range of %d..%d',
     [Index, 0, i]);

  Longint(P) := Longint(FMemory) + SizeOf(Word) * Index;
  Result := P^;
end;

procedure TCommonMemory.SetStr(const Value: String);
var
  n: Integer;
begin
  FStr := Value; // f[^̕\
  n := Length(FStr) + 4; // TCY4oCgĂB
  if n > FSize then n := FSize;
  ZeroMemory(FMemory, n);
  CopyMemory(FMemory, PChar(FStr), n - 4);
end;

procedure TCommonMemory.SetVByte(Index: Integer; const Value: Byte);
var
  P: PByte;
begin
  if (Index < 0) or (Index >= FSize) then
    raise ERangeError.CreateFmt('Index = %d is not within the valid range of %d..%d',
     [Index, 0, FSize - 1]);

  Longint(P) := Longint(FMemory) + Index;
  P^ := Value;
end;

procedure TCommonMemory.SetVDouble(Index: Integer; const Value: Double);
var
  i: Integer;
  P: PDouble;
begin
  i := FSize div SizeOf(Double);
  if (Index < 0) or (Index > i) then
    raise ERangeError.CreateFmt('Index = %d is not within the valid range of %d..%d',
     [Index, 0, i]);

  Longint(P) := Longint(FMemory) + SizeOf(Double) * Index;
  P^ := Value;
end;

procedure TCommonMemory.SetVInteger(Index: Integer; const Value: Integer);
var
  i: Integer;
  P: PInteger;
begin
  i := FSize div SizeOf(Integer);
  if (Index < 0) or (Index > i) then
    raise ERangeError.CreateFmt('Index = %d is not within the valid range of %d..%d',
     [Index, 0, i]);

  Longint(P) := Longint(FMemory) + SizeOf(Integer) * Index;
  P^ := Value;
end;

procedure TCommonMemory.SetVWord(Index: Integer; const Value: Word);
var
  i: Integer;
  P: PWord;
begin
  i := FSize div SizeOf(Word);
  if (Index < 0) or (Index > i) then
    raise ERangeError.CreateFmt('Index = %d is not within the valid range of %d..%d',
     [Index, 0, i]);

  Longint(P) := Longint(FMemory) + SizeOf(Word) * Index;
  P^ := Value;
end;

end.

